{ config, lib, pkgs, ... }: let
  serverConfig = pkgs.writeText "server.toml" ''
    bindaddress = "[::]:8443"
    db_path = "/data/kanidm.db"
    tls_chain = "/data/chain.pem"
    tls_key = "/data/key.pem"
    domain = "idm.obscuresecure.dev"
    origin = "https://idm.obscuresecure.dev"
    trust_x_forward_for = true
    [online_backup]
    path = "/data/kanidm/backups/"
    schedule = "00 22 * * *"
  '';

in {
  imports = [
    ./hardware-configuration.nix
    ../../../modules/common_vm.nix
  ];

  boot.loader.grub.enable = true;
  boot.loader.grub.device = "/dev/sda";
  boot.loader.grub.useOSProber = true;

  networking.hostName = "kanidm";
  networking.firewall.allowedTCPPorts = [ 80 443 ];
  virtualisation.docker.enable = true;

  security.acme = {
    acceptTerms = true;
    certs = {
      "idm.obscuresecure.dev" = {
        email = "admin@zettoit.eu";
        listenHTTP = "[::1]:8080";
      };
    };
  };

  services.traefik = {
    enable = true;
    staticConfigOptions = {
      entryPoints = {
        web.address = ":80";
      };
    };
    dynamicConfigOptions = {
      http = {
        routers = {
          acme = {
            rule = "PathPrefix(`/.well-known/acme-challenge`)";
            entryPoints = [ "web" ];
            service = "acme";
          };
          kanidm = {
            rule = "PathPrefix(`/`)";
            entryPoints = [ "web" ];
            middlewares = [ "redirect-to-https" ];
            service = "empty";
          };
        };
        middlewares.redirect-to-https.redirectScheme = {
          scheme = "https";
          permanent = true;
        };
        services = {
          empty.loadBalancer.servers = [];
          acme.loadBalancer.servers = [{
            url = "http://[::1]:8080";
          }];
        };
      };
    };
  };

  virtualisation.arion = {
    backend = "docker";
    projects.kanidm.settings = {
      docker-compose.volumes = {
        kanidm = {};
      };
      services = {
        kanidm_server.service = {
          image = "kanidm/server:latest";
          volumes = [
            "${serverConfig}:/data/server.toml"
            "/var/lib/acme/idm.obscuresecure.dev/fullchain.pem:/data/chain.pem"
            "/var/lib/acme/idm.obscuresecure.dev/key.pem:/data/key.pem"
            "kanidm:/data"
          ];
          ports = [ "443:8443" ];
        };
      };
    };
  };
  # This value determines the NixOS release from which the default
  # settings for stateful data, like file locations and database versions
  # on your system were taken. It‘s perfectly fine and recommended to leave
  # this value at the release version of the first install of this system.
  # Before changing this value read the documentation for this option
  # (e.g. man configuration.nix or on https://nixos.org/nixos/options.html).
  system.stateVersion = "23.05"; # Did you read the comment?
}
